from numpy import zeros

from gwlfe.Input.LandUse.Ag.GwAgLE import GwAgLE
from gwlfe.Input.LandUse.Ag.GwAgLE import GwAgLE_f
from gwlfe.Memoization import memoize


@memoize
def TileDrainGW(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                Landuse, TileDrainDensity):
    result = zeros((NYrs, 12))
    gwagle = GwAgLE(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                    Landuse)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (result[Y][i] + [gwagle[Y][i] * TileDrainDensity])
    return result


@memoize
def TileDrainGW_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                  Landuse, TileDrainDensity):
    if (TileDrainDensity > 0):
        gwagle = GwAgLE_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                          Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                          SeepCoef, Landuse)
        return gwagle * TileDrainDensity
    else:
        return zeros((NYrs, 12))



from numpy import zeros

from .InitNgN import InitNgN
from .InitNgN import InitNgN_f
from gwlfe.Memoization import memoize


def NGAppManN(NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = zeros((12,))
    init_ng_n = InitNgN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result[i] = NGPctManApp[i] * init_ng_n
    return result


@memoize
def NGAppManN_f(NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    return NGPctManApp * InitNgN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)



from unittest import skip

import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import SatStorCarryOver


class TestSatStorCarryOver(VariableUnitTest):
    @skip("not ready")
    def test_SatStorCarryOver(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SatStorCarryOver.SatStorCarryOver_2(),
            SatStorCarryOver.SatStorCarryOver(), decimal=7)



import math

from numpy import exp
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.QrunP import QrunP
from gwlfe.MultiUse_Fxns.Runoff.QrunP import QrunP_f

try:
    from .WashPerv_inner_compiled import WashPerv_inner
except ImportError:
    print("Unable to import compiled WashPerv_inner, using slower version")
    from gwlfe.MultiUse_Fxns.Runoff.WashPerv_inner import WashPerv_inner


@memoize
def WashPerv(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNP_0, AntMoist_0, Grow_0, NRur, NUrb):
    pervaccum = zeros(16)
    nlu = NLU(NRur, NUrb)
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    qrunp = QrunP(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)
    washperv = zeros((NYrs, 12, 31, 16))
    carryover = zeros(16)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                for l in range(nlu):
                    pervaccum[l] = carryover[l]
                    pervaccum[l] = (pervaccum[l] * exp(-0.12) + (1 / 0.12) * (1 - exp(-0.12)))
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        for l in range(NRur, nlu):
                            washperv[Y][i][j][l] = (1 - math.exp(-1.81 * qrunp[Y][i][j][l])) * pervaccum[l]
                            pervaccum[l] -= washperv[Y][i][j][l]
                else:
                    pass
                for l in range(nlu):
                    carryover[l] = pervaccum[l]
    return washperv


def WashPerv_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, CNP_0, AntMoist_0, Grow_0, NRur, NUrb):
    nlu = NLU(NRur, NUrb)
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    qrunp = QrunP_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, CNP_0, AntMoist_0, Grow_0)
    return WashPerv_inner(NYrs, DaysMonth, Temp, NRur, nlu, water, qrunp)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.Stream import SEDSTAB


class TestSEDSTAB(VariableUnitTest):
    def test_SEDSTAB(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SEDSTAB.SEDSTAB_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                              z.AntMoist_0, z.Grow_0,
                              z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                              z.MaxWaterCap, z.SatStor_0,
                              z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                              z.TileDrainDensity, z.PointFlow,
                              z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                              z.SedAFactor_0, z.AvKF,
                              z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d),
            SEDSTAB.SEDSTAB(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                            z.AntMoist_0, z.Grow_0,
                            z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs,
                            z.MaxWaterCap, z.SatStor_0,
                            z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                            z.TileDrainDensity, z.PointFlow,
                            z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                            z.SedAFactor_0, z.AvKF,
                            z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d), decimal=7)



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


class YesOrNo(object):
    NO = '<No>'
    YES = '<Yes>'

    @classmethod
    def parse(cls, value):
        if value in ('0', 'N'):
            return cls.NO
        elif value in ('1', 'Y'):
            return cls.YES
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def intval(cls, value):
        if value == cls.NO:
            return 0
        elif value == cls.YES:
            return 1
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def gmsval(cls, value):
        if value == cls.NO:
            return 'N'
        elif value == cls.YES:
            return 'Y'
        raise ValueError('Unexpected value: ' + str(value))


class ETflag(object):
    HAMON_METHOD = '<Hamon method>'
    BLAINY_CRIDDLE_METHOD = '<Blainy-Criddle method>'

    @classmethod
    def parse(cls, value):
        value = int(value)
        if value == 0:
            return cls.HAMON_METHOD
        elif value == 1:
            return cls.BLAINY_CRIDDLE_METHOD
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def gmsval(cls, value):
        if value == cls.HAMON_METHOD:
            return 0
        elif value == cls.BLAINY_CRIDDLE_METHOD:
            return 1
        raise ValueError('Unexpected value: ' + str(value))


GROWING_SEASON = '<Growing season>'


class GrowFlag(object):
    NON_GROWING_SEASON = '<Non-growing season>'
    GROWING_SEASON = '<Growing season>'

    @classmethod
    def parse(cls, value):
        value = int(value)
        if value == 0:
            return cls.NON_GROWING_SEASON
        elif value == 1:
            return cls.GROWING_SEASON
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def intval(cls, value):
        if value == cls.NON_GROWING_SEASON:
            return 0
        elif value == cls.GROWING_SEASON:
            return 1
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def gmsval(cls, value):
        return cls.intval(value)


class SweepType(object):
    VACUUM = '<Vacuum>'
    MECHANICAL = '<Mechanical>'

    @classmethod
    def parse(cls, value):
        value = int(value)
        if value == 1:
            return cls.MECHANICAL
        elif value == 2:
            return cls.VACUUM
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def gmsval(cls, value):
        if value == cls.MECHANICAL:
            return 1
        elif value == cls.VACUUM:
            return 2
        raise ValueError('Unexpected value: ' + str(value))


# TODO: Use actual NLCD values
# Reference: https://drive.google.com/a/azavea.com/file/d/0B3v0QxIOuR_nX3Rnekp0NGUyOGM/view
class LandUse(object):
    WATER = '<Water>'
    HAY_PAST = '<Hay/Past>'
    CROPLAND = '<Cropland>'
    FOREST = '<Forest>'
    WETLAND = '<Wetland>'
    DISTURBED = '<Disturbed>'
    TURFGRASS = '<Turfgrass>'
    OPEN_LAND = '<Open_Land>'
    BARE_ROCK = '<Bare_Rock>'
    SANDY_AREAS = '<Sandy_Areas>'
    UNPAVED_ROAD = '<Unpaved_Road>'
    LD_MIXED = '<Ld_Mixed>'
    MD_MIXED = '<Md_Mixed>'
    HD_MIXED = '<Hd_Mixed>'
    LD_RESIDENTIAL = '<Ld_Residential>'
    MD_RESIDENTIAL = '<Md_Residential>'
    HD_RESIDENTIAL = '<Hd_Residential>'

    @classmethod
    def parse(cls, value):
        if value == 'Water':
            return cls.WATER
        if value in ('Hay/Past', 'Hay'):
            return cls.HAY_PAST
        elif value == 'Cropland':
            return cls.CROPLAND
        elif value == 'Forest':
            return cls.FOREST
        elif value == 'Wetland':
            return cls.WETLAND
        elif value in ('Disturbed', 'Disturbed Land'):
            return cls.DISTURBED
        elif value == 'Turfgrass':
            return cls.TURFGRASS
        elif value in ('Open_Land', 'Open Land'):
            return cls.OPEN_LAND
        elif value in ('Bare_Rock', 'Bare Rock'):
            return cls.BARE_ROCK
        elif value in ('Sandy_Areas', 'Sandy Areas'):
            return cls.SANDY_AREAS
        elif value in ('Unpaved_Road', 'Unpaved Roads'):
            return cls.UNPAVED_ROAD
        elif value in ('Ld_Mixed', 'LD Mixed'):
            return cls.LD_MIXED
        elif value in ('Md_Mixed', 'MD Mixed'):
            return cls.MD_MIXED
        elif value in ('Hd_Mixed', 'HD Mixed'):
            return cls.HD_MIXED
        elif value in ('Ld_Residential', 'LD Residential',
                       'Ld_Open_Space', 'LD Open Space'):
            return cls.LD_RESIDENTIAL
        elif value in ('Md_Residential', 'MD Residential'):
            return cls.MD_RESIDENTIAL
        elif value in ('Hd_Residential', 'HD Residential'):
            return cls.HD_RESIDENTIAL
        raise ValueError('Unexpected value: ' + str(value))

    @classmethod
    def gmsval(cls, value):
        if value == cls.WATER:
            return 'Water'
        elif value == cls.HAY_PAST:
            return 'Hay/Past'
        elif value == cls.CROPLAND:
            return 'Cropland'
        elif value == cls.FOREST:
            return 'Forest'
        elif value == cls.WETLAND:
            return 'Wetland'
        elif value == cls.DISTURBED:
            return 'Disturbed'
        elif value == cls.TURFGRASS:
            return 'Turfgrass'
        elif value == cls.OPEN_LAND:
            return 'Open_Land'
        elif value == cls.BARE_ROCK:
            return 'Bare_Rock'
        elif value == cls.SANDY_AREAS:
            return 'Sandy_Areas'
        elif value == cls.UNPAVED_ROAD:
            return 'Unpaved_Road'
        elif value == cls.LD_MIXED:
            return 'Ld_Mixed'
        elif value == cls.MD_MIXED:
            return 'Md_Mixed'
        elif value == cls.HD_MIXED:
            return 'Hd_Mixed'
        elif value == cls.LD_RESIDENTIAL:
            return 'Ld_Open_Space'
        elif value == cls.MD_RESIDENTIAL:
            return 'Md_Residential'
        elif value == cls.HD_RESIDENTIAL:
            return 'Hd_Residential'
        raise ValueError('Unexpected value: ' + str(value))



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import AvCNUrb


class TestAvCNUrb(VariableUnitTest):
    # @skip("Not Ready Yet.")
    def test_AvCNUrb(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvCNUrb.AvCNUrb_f(z.NRur, z.NUrb, z.CNI_0, z.CNP_0, z.Imper, z.Area),
            AvCNUrb.AvCNUrb(z.NRur, z.NUrb, z.CNI_0, z.CNP_0, z.Imper, z.Area), decimal=7)



#!/usr/bin/env python

from codecs import open
from os import path

from setuptools import setup, find_packages

from gwlfe.Input.WaterBudget.AMC5_yesterday_inner import cc as amc5ycc
# Added to fix error.
# See http://stackoverflow.com/questions/9352656/python-assertionerror-when-running-nose-tests-with-coverage  # NOQA
from gwlfe.MultiUse_Fxns.Discharge.AdjUrbanQTotal_inner import cc as adjcc  # TODO: use full length names
from gwlfe.MultiUse_Fxns.Runoff.CNumImperv_inner import cc as cnicc
from gwlfe.MultiUse_Fxns.Runoff.CNumPerv_inner import cc as cnpcc
from gwlfe.MultiUse_Fxns.Runoff.CNum_inner import cc as cncc
from gwlfe.Input.WaterBudget.DeepSeep_inner import cc as dscc
from gwlfe.Input.WaterBudget.InitSnowYesterday_inner import cc as isycc
from gwlfe.Input.WaterBudget.InitSnow_inner import cc as iscc
from gwlfe.Input.WaterBudget.Percolation_inner import cc as pcc
from gwlfe.Input.WaterBudget.UnsatStor_inner import cc as usscc
from gwlfe.BMPs.Stream.UrbLoadRed_inner import cc as ulrcc
from gwlfe.MultiUse_Fxns.Runoff.WashImperv_inner import cc as wipcc
from gwlfe.MultiUse_Fxns.Runoff.WashPerv_inner import cc as wpcc

# Get the long description from README.md
with open(path.join(path.abspath(path.dirname(__file__)),
                    'README.md'), encoding='utf-8') as f:
    long_description = f.read()

tests_require = [
    'pynose == 1.5.*',
    'coverage == 4.0.3'
]

setup(
    name='gwlf-e',
    version='3.2.0',
    description='A Python port of Generalized Watersheds Loading Functions - Enhanced (MapShed)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/WikiWatershed/gwlf-e',
    author='Azavea Inc.',
    author_email='systems@azavea.com',
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
    ],
    keywords='gwlf-e watershed hydrology',
    packages=find_packages(exclude=['test']),
    python_requires=">=3.7",
    install_requires=[
        'certifi>=2021',
        'funcsigs==1.0.2',
        'llvmlite==0.38.*',
        'pynose==1.5.*',
        'numpy==1.21.*',
        'numba==0.55.*',
    ],
    extras_require={
        'dev': [],
        'test': tests_require,
    },
    test_suite='nose.collector',
    tests_require=tests_require,
    ext_modules=[adjcc.distutils_extension(), amc5ycc.distutils_extension(), cncc.distutils_extension(),
                 cnicc.distutils_extension(), cnpcc.distutils_extension(), dscc.distutils_extension(),
                 iscc.distutils_extension(), isycc.distutils_extension(), pcc.distutils_extension(),
                 usscc.distutils_extension(), ulrcc.distutils_extension(), wpcc.distutils_extension(),
                 wipcc.distutils_extension()]
)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import GroundWatLE


class TestGroundWatLE(VariableUnitTest):

    def test_GroundWatLE_ground_truth(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/GroundWatLE.npy"),
            GroundWatLE.GroundWatLE(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                    z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                    z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                    z.RecessionCoef, z.SeepCoef), decimal=7)

    def test_GroundWatLE(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GroundWatLE.GroundWatLE_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                      z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                      z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                      z.RecessionCoef, z.SeepCoef),
            GroundWatLE.GroundWatLE(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                    z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN,
                                    z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                    z.RecessionCoef, z.SeepCoef), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Discharge import UrbanQTotal_1


class TestUrbanQTotal_1(VariableUnitTest):

    def test_UrbanQTotal_1_ground_truth(self):
        z = self.z
        np.testing.assert_array_almost_equal(

            np.load(self.basepath + "/UrbanQTotal_1.npy"),
            UrbanQTotal_1.UrbanQTotal_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur,
                                        z.NUrb, z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR,
                                        z.ISRA), decimal=7)

    # @skip("not ready")
    def test_UrbanQTotal_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            UrbanQTotal_1.UrbanQTotal_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur,
                                          z.NUrb, z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR,
                                          z.ISRA),
            UrbanQTotal_1.UrbanQTotal_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur,
                                        z.NUrb, z.Area, z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR,
                                        z.ISRA), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import NConc


class TestNConc(VariableUnitTest):
    def test_NConc(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NConc.NConc_f(z.NRur, z.NUrb, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                          z.FirstManureMonth2, z.LastManureMonth2),
            NConc.NConc(z.NRur, z.NUrb, z.NitrConc, z.ManNitr, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                        z.FirstManureMonth2, z.LastManureMonth2), decimal=7)



from numpy import zeros

from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff
from gwlfe.MultiUse_Fxns.Runoff.RurQRunoff import RurQRunoff_f
from gwlfe.Output.Loading.PConc import PConc
from gwlfe.Output.Loading.PConc import PConc_f


@memoize
def pRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc, ManuredAreas,
            FirstManureMonth, LastManureMonth, ManPhos, FirstManureMonth2,
            LastManureMonth2):
    result = zeros((NYrs, 12, 10))
    rur_q_runoff = RurQRunoff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0)
    p_conc = PConc(NRur, NUrb, PhosConc, ManPhos, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2,
                   LastManureMonth2)
    for Y in range(NYrs):
        for i in range(12):
            for l in range(NRur):
                # += changed to =
                result[Y][i][l] = 0.1 * p_conc[i][l] * rur_q_runoff[Y][l][i] * Area[l]
    return result


@memoize
def pRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0, Area, PhosConc, ManuredAreas,
              FirstManureMonth, LastManureMonth, ManPhos, FirstManureMonth2, LastManureMonth2):
    p_conc = PConc_f(NRur, NUrb, PhosConc, ManPhos, ManuredAreas, FirstManureMonth, LastManureMonth, FirstManureMonth2,
                     LastManureMonth2)[:, :NRur]
    rur_q_runoff = RurQRunoff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0)
    return 0.1 * p_conc * rur_q_runoff * Area[:NRur]



from numpy import array
from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.Ag.AgAreaTotal import AgAreaTotal
# from Timer import time_function
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.Qrun import Qrun, Qrun_f
from gwlfe.MultiUse_Fxns.Runoff.Retention import Retention
from gwlfe.enums import LandUse


@memoize
def AgQTotal(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse, Area):
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    retention = Retention(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)
    q_run = Qrun(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)
    ag_area_total = AgAreaTotal(NRur, Landuse, Area)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = 0

                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        if CN[l] > 0:
                            if water[Y][i][j] >= 0.2 * retention[Y][i][j][l]:
                                if Landuse[l] is LandUse.CROPLAND:
                                    # (Maybe used for STREAMPLAN?)
                                    result[Y][i][j] += q_run[Y][i][j][l] * Area[l]
                                    # z.AgQRunoff[l][i] += z.Qrun[Y][i][j][l]
                                elif Landuse[l] is LandUse.HAY_PAST:
                                    result[Y][i][j] += q_run[Y][i][j][l] * Area[l]
                                    # z.AgQRunoff[l][i] += z.Qrun[Y][i][j][l]
                                elif Landuse[l] is LandUse.TURFGRASS:
                                    result[Y][i][j] += q_run[Y][i][j][l] * Area[l]
                                    # z.AgQRunoff[l][i] += z.Qrun[Y][i][j][l]
                    if ag_area_total > 0:
                        result[Y][i][j] = result[Y][i][j] / ag_area_total
                    else:
                        result[Y][i][j] = 0
    return result


@memoize
def AgQTotal_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse, Area):
    result = zeros((NYrs, 12, 31))
    q_run = Qrun_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)
    ag_area_total = AgAreaTotal(NRur, Landuse, Area)
    ag_used = array([1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
    ag_area = Area * ag_used
    qrun_agarea = q_run * ag_area
    if ag_area_total > 0:
        result = sum(qrun_agarea, axis=3) / ag_area_total
    return result



# -*- coding: utf-8 -*-
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .Input.LandUse.TotAreaMeters import TotAreaMeters
from .MultiUse_Fxns.Discharge.StreamFlow_1 import StreamFlow_1_f
from .MultiUse_Fxns.LossFactAdj import LossFactAdj_f
from .MultiUse_Fxns.Runoff.UrbRunoffLiter import UrbRunoffLiter_f

"""
Imported from AFOS.bas
"""

import logging

log = logging.getLogger(__name__)


def AnimalOperations(z, Y):
    for i in range(12):
        z.NGLostManP[Y][i] = (z.NGAppManP[i] * z.NGAppPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                              * (1 - z.NGPctSoilIncRate[i]))

        if z.NGLostManP[Y][i] > z.NGAppManP[i]:
            z.NGLostManP[Y][i] = z.NGAppManP[i]
        if z.NGLostManP[Y][i] < 0:
            z.NGLostManP[Y][i] = 0

        z.NGLostManFC[Y][i] = (z.NGAppManFC[i] * z.NGAppFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                               * (1 - z.NGPctSoilIncRate[i]))

        if z.NGLostManFC[Y][i] > z.NGAppManFC[i]:
            z.NGLostManFC[Y][i] = z.NGAppManFC[i]
        if z.NGLostManFC[Y][i] < 0:
            z.NGLostManFC[Y][i] = 0

        z.NGLostBarnP[Y][i] = (z.NGInitBarnP[i] * z.NGBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                               - z.NGInitBarnP[i] * z.NGBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                   i] * z.AWMSNgPct * z.NgAWMSCoeffP
                               + z.NGInitBarnP[i] * z.NGBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                   i] * z.RunContPct * z.RunConCoeffP)

        if z.NGLostBarnP[Y][i] > z.NGInitBarnP[i]:
            z.NGLostBarnP[Y][i] = z.NGInitBarnP[i]
        if z.NGLostBarnP[Y][i] < 0:
            z.NGLostBarnP[Y][i] = 0

        z.NGLostBarnFC[Y][i] = (z.NGInitBarnFC[i] * z.NGBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                                - z.NGInitBarnFC[i] * z.NGBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                    i] * z.AWMSNgPct * z.NgAWMSCoeffP
                                + z.NGInitBarnFC[i] * z.NGBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                    i] * z.RunContPct * z.RunConCoeffP)

        if z.NGLostBarnFC[Y][i] > z.NGInitBarnFC[i]:
            z.NGLostBarnFC[Y][i] = z.NGInitBarnFC[i]
        if z.NGLostBarnFC[Y][i] < 0:
            z.NGLostBarnFC[Y][i] = 0

        # Grazing animal losses

        z.GRLostManP[Y][i] = (z.GRAppManP[i] * z.GRAppPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                              * (1 - z.GRPctSoilIncRate[i]))

        if z.GRLostManP[Y][i] > z.GRAppManP[i]:
            z.GRLostManP[Y][i] = z.GRAppManP[i]
        if z.GRLostManP[Y][i] < 0:
            z.GRLostManP[Y][i] = 0

        z.GRLostManFC[Y][i] = (z.GRAppManFC[i] * z.GRAppFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                               * (1 - z.GRPctSoilIncRate[i]))

        if z.GRLostManFC[Y][i] > z.GRAppManFC[i]:
            z.GRLostManFC[Y][i] = z.GRAppManFC[i]
        if z.GRLostManFC[Y][i] < 0:
            z.GRLostManFC[Y][i] = 0

        z.GRLostBarnP[Y][i] = (z.GRInitBarnP[i] * z.GRBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                               - z.GRInitBarnP[i] * z.GRBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                   i] * z.AWMSGrPct * z.GrAWMSCoeffP
                               + z.GRInitBarnP[i] * z.GRBarnPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                   i] * z.RunContPct * z.RunConCoeffP)

        if z.GRLostBarnP[Y][i] > z.GRInitBarnP[i]:
            z.GRLostBarnP[Y][i] = z.GRInitBarnP[i]
        if z.GRLostBarnP[Y][i] < 0:
            z.GRLostBarnP[Y][i] = 0

        z.GRLostBarnFC[Y][i] = (z.GRInitBarnFC[i] * z.GRBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i]
                                - z.GRInitBarnFC[i] * z.GRBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                    i] * z.AWMSGrPct * z.GrAWMSCoeffP
                                + z.GRInitBarnFC[i] * z.GRBarnFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][
                                    i] * z.RunContPct * z.RunConCoeffP)

        if z.GRLostBarnFC[Y][i] > z.GRInitBarnFC[i]:
            z.GRLostBarnFC[Y][i] = z.GRInitBarnFC[i]
        if z.GRLostBarnFC[Y][i] < 0:
            z.GRLostBarnFC[Y][i] = 0

        z.GRLossP[Y][i] = ((z.GrazingP[i] - z.GRStreamP[i])
                           * z.GrazingPRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i])

        if z.GRLossP[Y][i] > (z.GrazingP[i] - z.GRStreamP[i]):
            z.GRLossP[Y][i] = (z.GrazingP[i] - z.GRStreamP[i])
        if z.GRLossP[Y][i] < 0:
            z.GRLossP[Y][i] = 0

        z.GRLossFC[Y][i] = ((z.GrazingFC[i] - z.GRStreamFC[i])
                            * z.GrazingFCRate[i] * LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i])

        if z.GRLossFC[Y][i] > (z.GrazingFC[i] - z.GRStreamFC[i]):
            z.GRLossFC[Y][i] = (z.GrazingFC[i] - z.GRStreamFC[i])
        if z.GRLossFC[Y][i] < 0:
            z.GRLossFC[Y][i] = 0

        # Total animal related losses

        z.AnimalP[Y][i] = ((z.NGLostManP[Y][i]
                            + z.GRLostManP[Y][i]
                            + z.NGLostBarnP[Y][i]
                            + z.GRLostBarnP[Y][i]
                            + z.GRLossP[Y][i]
                            + z.GRStreamP[i])
                           - ((z.NGLostManP[Y][i] + z.NGLostBarnP[Y][i]) * z.PhytasePct * z.PhytaseCoeff))

        z.AnimalFC[Y][i] = (z.NGLostManFC[Y][i]
                            + z.GRLostManFC[Y][i]
                            + z.NGLostBarnFC[Y][i]
                            + z.GRLostBarnFC[Y][i]
                            + z.GRLossFC[Y][i]
                            + z.GRStreamFC[i])

        # CACULATE PATHOGEN LOADS
        z.ForestAreaTotalSqMi = 0
        z.ForestAreaTotalSqMi = (z.ForestAreaTotal * 0.01) / 2.59

        z.PtFlowLiters = (z.PointFlow[i] / 100) * TotAreaMeters(z.NRur, z.NUrb, z.Area) * 1000

        # Get the wildlife orgs
        z.WWOrgs[Y][i] = z.PtFlowLiters * (z.WWTPConc * 10) * (1 - z.InstreamDieoff)
        z.SSOrgs[Y][i] = (z.SepticOrgsDay
                          * z.SepticsDay[i]
                          * z.DaysMonth[Y][i]
                          * z.SepticFailure
                          * (1 - z.InstreamDieoff))

        if LossFactAdj_f(z.Prec, z.DaysMonth)[Y][i] * (1 - z.WuDieoff) > 1:
            z.UrbOrgs[Y][i] = (z.UrbRunoffLiter[Y][i]
                               * (z.UrbEMC * 10)
                               * (1 - z.InstreamDieoff))
            z.WildOrgs[Y][i] = (z.WildOrgsDay
                                * z.DaysMonth[Y][i]
                                * z.WildDensity
                                * z.ForestAreaTotalSqMi
                                * (1 - z.InstreamDieoff))
        else:
            z.UrbOrgs[Y][i] = (
                    UrbRunoffLiter_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.Area,
                                     z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA)[Y][i]
                    * (z.UrbEMC * 10)
                    * (1 - z.WuDieoff)
                    * (1 - z.InstreamDieoff))
            z.WildOrgs[Y][i] = (z.WildOrgsDay
                                * z.DaysMonth[Y][i]
                                * z.WildDensity
                                * z.ForestAreaTotalSqMi
                                * (1 - z.WuDieoff)
                                * (1 - z.InstreamDieoff))

        # Get the total orgs
        z.TotalOrgs[Y][i] = (z.WWOrgs[Y][i]
                             + z.SSOrgs[Y][i]
                             + z.UrbOrgs[Y][i]
                             + z.WildOrgs[Y][i]
                             + z.AnimalFC[Y][i])

        z.CMStream[Y][i] = (StreamFlow_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area,
                                           z.CNI_0, z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper,
                                           z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET, z.DayHrs, z.MaxWaterCap,
                                           z.SatStor_0, z.RecessionCoef, z.SeepCoef)[Y][i] / 100) * TotAreaMeters(
            z.NRur, z.NUrb, z.Area)

        if z.CMStream[Y][i] > 0:
            z.OrgConc[Y][i] = (z.TotalOrgs[Y][i] / (z.CMStream[Y][i] * 1000)) / 10
        else:
            z.OrgConc[Y][i] = 0



from numpy import zeros

from gwlfe.Input.LandUse.Ag.TileDrainGW import TileDrainGW
from gwlfe.Input.LandUse.Ag.TileDrainGW import TileDrainGW_f
from gwlfe.Input.LandUse.Ag.TileDrainRO import TileDrainRO
from gwlfe.Input.LandUse.Ag.TileDrainRO import TileDrainRO_f
from gwlfe.Memoization import memoize


def TileDrain(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
              ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef, Landuse,
              TileDrainDensity):
    result = zeros((NYrs, 12))
    tiledrainro = TileDrainRO(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse,
                              Area,
                              TileDrainDensity)
    tiledraingw = TileDrainGW(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper,
                              ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef,
                              SeepCoef, Landuse, TileDrainDensity)
    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = (result[Y][i] + tiledrainro[Y][i] + tiledraingw[Y][i])
    return result


@memoize
def TileDrain_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0, RecessionCoef, SeepCoef,
                Landuse, TileDrainDensity):
    tiledrainro = TileDrainRO_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, AntMoist_0, NUrb, Grow_0, Landuse,
                                Area,
                                TileDrainDensity)
    tiledraingw = TileDrainGW_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                RecessionCoef, SeepCoef, Landuse, TileDrainDensity)
    return tiledrainro + tiledraingw



from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Input.LandUse.AreaTotal import AreaTotal_f
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal_f


def PcntUrbanArea(NRur, NUrb, Area):
    result = 0
    areatotal = AreaTotal(NRur, NUrb, Area)
    urbareatotal = UrbAreaTotal(NRur, NUrb, Area)
    if areatotal == 0:
        result = 0
    else:
        result += urbareatotal / areatotal
    return result


def PcntUrbanArea_f(NRur, NUrb, Area):
    areatotal = AreaTotal_f(Area)
    if areatotal != 0:
        return UrbAreaTotal_f(NRur, NUrb, Area) / areatotal
    else:
        return 0



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import Load


class TestLoad(VariableUnitTest):

    def test_Load(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Load.Load_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0,
                        z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual,
                        z.LoadRateImp,
                        z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth, z.PctStrmBuf),
            Load.Load(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0, z.AntMoist_0,
                      z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual, z.LoadRateImp,
                      z.SweepFrac, z.UrbSweepFrac, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.DisFract, z.FilterWidth,
                      z.PctStrmBuf), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import LuRunoff


class TestLuRunoff(VariableUnitTest):

    def test_LuRunoff(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            LuRunoff.LuRunoff_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                                z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA, z.CN),
            LuRunoff.LuRunoff(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.NRur, z.NUrb, z.CNI_0, z.CNP_0,
                              z.AntMoist_0, z.Grow_0, z.Imper, z.ISRR, z.ISRA, z.CN), decimal=7)



from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import AvGRStreamN
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import AvGRStreamN_f


def NFENCING(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, n42, n45, n69):
    grsn = AvGRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    if n42 > 0:  # based on the code, n42 is always > 0 (may not need to check
        result = (n45 / n42) * n69 * grsn
    else:
        result = 0  # TODO: the code does not have this fall back, would have error if n42 <= 0
    return result


def NFENCING_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, n42, n45, n69):
    if n42 > 0:  # based on the code, n42 is always > 0 (may not need to check
        grsn = AvGRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
        return (n45 / n42) * n69 * grsn
    else:
        return 0  # TODO: the code does not have this fall back, would have error if n42 <= 0



from numpy import zeros

from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Input.LandUse.Urb.UrbAreaTotal import UrbAreaTotal
# from Timer import time_function
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal import UrbanQTotal, UrbanQTotal_f


@memoize
def UrbanQTotal_1(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                  ISRR, ISRA):
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urban_area_total = UrbAreaTotal(NRur, NUrb, Area)
    area_total = AreaTotal(NRur, NUrb, Area)
    urban_q_total = UrbanQTotal(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0, Imper, ISRR, ISRA)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if urban_area_total > 0:
                        result[Y][i][j] = urban_q_total[Y][i][j] * urban_area_total / area_total
                    else:
                        result[Y][i][j] = 0
    return result


@memoize
def UrbanQTotal_1_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                    ISRR, ISRA):
    urban_area_total = UrbAreaTotal(NRur, NUrb, Area)
    area_total = AreaTotal(NRur, NUrb, Area)
    urban_q_total = UrbanQTotal_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                                  CNP_0, Imper, ISRR, ISRA)
    return urban_q_total * urban_area_total / area_total



from numpy import average
from numpy import zeros

from gwlfe.Input.WaterBudget.Evapotrans import Evapotrans
from gwlfe.Input.WaterBudget.Evapotrans import Evapotrans_f
from gwlfe.Memoization import memoize


@memoize
def AvEvapoTrans(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0, Imper,
                 ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, ETFlag):
    result = zeros(12)
    evapotrans = Evapotrans(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                            Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, ETFlag)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += evapotrans[Y][i] / NYrs
    return result


@memoize
def AvEvapoTrans_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0, CNP_0,
                   Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap):
    evapotrans = Evapotrans_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap)
    return average(evapotrans, axis=0)



# from Timer import time_function
from gwlfe.Memoization import memoize


@memoize
def SedDelivRatio(SedDelivRatio_0):
    if SedDelivRatio_0 == 0:
        result = 0.0001
    else:
        result = SedDelivRatio_0
    return result

# def SedDelivRatio_f():
#     pass



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.Stream import SEDFEN


class TestSEDFEN(VariableUnitTest):

    def test_SEDFEN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SEDFEN.SEDFEN_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                            z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV,
                            z.PcntET,
                            z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention,
                            z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal,
                            z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                            z.AvSlope, z.SedAAdjust, z.StreamLength, z.AgLength, z.n42, z.n45, z.n85),
            SEDFEN.SEDFEN(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                          z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                          z.DayHrs, z.MaxWaterCap, z.SatStor_0, z.RecessionCoef, z.SeepCoef, z.Qretention,
                          z.PctAreaInfil, z.n25b, z.Landuse, z.TileDrainDensity, z.PointFlow, z.StreamWithdrawal,
                          z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj, z.SedAFactor_0, z.AvKF,
                          z.AvSlope, z.SedAAdjust, z.StreamLength, z.AgLength, z.n42, z.n45, z.n85), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import PConc


class TestPConc(VariableUnitTest):
    def test_PConc(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            PConc.PConc_f(z.NRur, z.NUrb, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                          z.FirstManureMonth2,
                          z.LastManureMonth2),
            PConc.PConc(z.NRur, z.NUrb, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth, z.LastManureMonth,
                        z.FirstManureMonth2,
                        z.LastManureMonth2), decimal=7)



from numpy import sum
from numpy import zeros

from .AnimalN import AnimalN
from .AnimalN import AnimalN_f
from gwlfe.Memoization import memoize


def AvAnimalN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
              NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN,
              RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate):
    result = zeros((12,))
    animal_n = AnimalN(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec,
                       DaysMonth,
                       NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct,
                       NgAWMSCoeffN,
                       RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams,
                       GrazingNRate)
    for Y in range(NYrs):
        for i in range(12):
            result[i] += animal_n[Y][i] / NYrs
    return result


@memoize
def AvAnimalN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec, DaysMonth,
                NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct, NgAWMSCoeffN,
                RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate):
    return sum(AnimalN_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, Prec,
                         DaysMonth,
                         NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate, NGBarnNRate, AWMSNgPct,
                         NgAWMSCoeffN,
                         RunContPct, RunConCoeffN, PctGrazing, GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams,
                         GrazingNRate), axis=0) / NYrs



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.InitSnow import InitSnow
from gwlfe.Input.WaterBudget.InitSnowYesterday import InitSnowYesterday
from gwlfe.Input.WaterBudget.Melt_1 import Melt_1, Melt_1_f
from gwlfe.Input.WaterBudget.Rain import Rain, Rain_f
from gwlfe.Memoization import memoize


@memoize
def Erosiv(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef):
    result = zeros((NYrs, 12, 31))
    init_snow = InitSnow(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    init_snow_yesterday = InitSnow_0
    rain = Rain(NYrs, DaysMonth, Temp, Prec)
    melt_1 = Melt_1(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0:
                    if (init_snow_yesterday > 0.001):
                        if rain[Y][i][j] > 0 and init_snow_yesterday - melt_1[Y][i][j] < 0.001:
                            result[Y][i][j] = 6.46 * Acoef[i] * rain[Y][i][j] ** 1.81
                    else:
                        if rain[Y][i][j] > 0 and init_snow_yesterday < 0.001:
                            result[Y][i][j] = 6.46 * Acoef[i] * rain[Y][i][j] ** 1.81

                init_snow_yesterday = init_snow[Y][i][j]
    return result


def Erosiv_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, Acoef):
    result = zeros((NYrs, 12, 31))
    rain = Rain_f(Temp, Prec)
    init_snow_yesterday = InitSnowYesterday(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    melt_1 = Melt_1_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    erosiv = 6.46 * Acoef.reshape((12, 1)) * rain ** 1.81
    result[where((Temp > 0) & (init_snow_yesterday > 0.001) & (rain > 0) & (init_snow_yesterday - melt_1 < 0.001))] = \
    erosiv[where((Temp > 0) & (init_snow_yesterday > 0.001) & (rain > 0) & (init_snow_yesterday - melt_1 < 0.001))]
    result[where((Temp > 0) & (init_snow_yesterday <= 0.001) & (rain > 0))] = erosiv[
        where((Temp > 0) & (init_snow_yesterday <= 0.001) & (rain > 0))]
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostBarnN import GRLostBarnN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLostBarnN import GRLostBarnN_f


class TestGRLostBarnN(VariableUnitTest):

    def test_GRLostBarnN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            GRLostBarnN(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                        z.GRPctManApp, z.PctGrazing, z.GRBarnNRate, z.Prec,
                        z.DaysMonth, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct, z.RunConCoeffN),
            GRLostBarnN_f(z.NYrs, z.Prec, z.DaysMonth, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                          z.GRPctManApp, z.PctGrazing, z.GRBarnNRate, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                          z.RunConCoeffN), decimal=7)



from unittest import skip

import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.AgAnimal import NAWMSP_2


class TestNAWMSP_2(VariableUnitTest):
    @skip('Not Ready Yet.')
    def test_NAWMSP_2(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NAWMSP_2.NAWMSP_2_f(),
            NAWMSP_2.NAWMSP_2(), decimal=7)



from numpy import where
from numpy import zeros

from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Input.WaterBudget.Water import Water_f
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal import UrbanQTotal
from gwlfe.MultiUse_Fxns.Discharge.UrbanQTotal import UrbanQTotal_f


# Precipitation.Precipitation(z.NYrs, z.DaysMonth, z.Prec)ize
def RetentionEff(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Qretention, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                 CNP_0,
                 Imper, ISRR, ISRA, PctAreaInfil):
    # result = 0
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urbanqtotal = UrbanQTotal(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                              CNP_0,
                              Imper, ISRR,
                              ISRA)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    if water[Y][i][j] < 0.05:
                        pass
                    else:
                        if Qretention > 0:
                            if urbanqtotal[Y][i][j] > 0:
                                if urbanqtotal[Y][i][j] <= Qretention * PctAreaInfil:
                                    result[Y][i][j] = 1
                                else:
                                    result[Y][i][j] = Qretention * PctAreaInfil / urbanqtotal[Y][i][j]
                else:
                    pass
    return result


@memoize
def RetentionEff_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, Qretention, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
                   CNP_0, Imper, ISRR, ISRA, PctAreaInfil):
    result = zeros((NYrs, 12, 31))
    water = Water_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    urbanqtotal = UrbanQTotal_f(NYrs, DaysMonth, NRur, NUrb, Temp, InitSnow_0, Prec, Area, CNI_0, AntMoist_0, Grow_0,
                                CNP_0,
                                Imper, ISRR, ISRA)
    result[where((Temp > 0) & (water > 0.05) & (Qretention > 0) & (urbanqtotal > 0) & (
            urbanqtotal <= Qretention * PctAreaInfil))] = 1
    result[where((Temp > 0) & (water > 0.05) & (Qretention > 0) & (urbanqtotal > 0) & (
            urbanqtotal > Qretention * PctAreaInfil))] = \
        Qretention * PctAreaInfil / urbanqtotal[where(
            (Temp > 0) & (water > 0.05) & (Qretention > 0) & (urbanqtotal > 0) & (
                    urbanqtotal > Qretention * PctAreaInfil))]

    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import LuTotPhos


class TestLuTotPhos(VariableUnitTest):
    def test_LuTotPhos(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            LuTotPhos.LuTotPhos_f(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                                  z.Grow_0, z.Area, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth,
                                  z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF,
                                  z.LS, z.C, z.P, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil,
                                  z.Nqual,
                                  z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth, z.PctStrmBuf,
                                  z.Acoef, z.SedPhos, z.CNI_0),
            LuTotPhos.LuTotPhos(z.NYrs, z.DaysMonth, z.InitSnow_0, z.Temp, z.Prec, z.AntMoist_0, z.NRur, z.NUrb, z.CN,
                                z.Grow_0, z.Area, z.PhosConc, z.ManPhos, z.ManuredAreas, z.FirstManureMonth,
                                z.LastManureMonth, z.FirstManureMonth2, z.LastManureMonth2, z.SedDelivRatio_0, z.KF,
                                z.LS, z.C, z.P, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.Qretention, z.PctAreaInfil, z.Nqual,
                                z.LoadRateImp, z.LoadRatePerv, z.Storm, z.UrbBMPRed, z.FilterWidth, z.PctStrmBuf,
                                z.Acoef, z.SedPhos, z.CNI_0), decimal=7)



from numpy import zeros

from gwlfe.Output.Loading.StreamBankEros import StreamBankEros
from gwlfe.Output.Loading.StreamBankEros import StreamBankEros_f


def SEDSTAB(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
            CNP_0, Imper,
            ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
            RecessionCoef, SeepCoef
            , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
            StreamWithdrawal, GroundWithdrawal
            , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
            SedAAdjust, StreamLength, n42b, n46c, n85d):
    result = zeros((NYrs, 12))
    streambankeros = StreamBankEros(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                    Grow_0,
                                    CNP_0, Imper,
                                    ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                    RecessionCoef, SeepCoef
                                    , Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
                                    StreamWithdrawal, GroundWithdrawal
                                    , NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope,
                                    SedAAdjust, StreamLength)
    for Y in range(NYrs):
        for i in range(12):
            if n42b > 0:
                result[Y][i] = (n46c / n42b) * streambankeros[Y][i] * n85d
    return result


def SEDSTAB_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0, Grow_0,
              CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse, TileDrainDensity, PointFlow,
              StreamWithdrawal, GroundWithdrawal, NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
              AvSlope, SedAAdjust, StreamLength, n42b, n46c, n85d):
    streambankeros = StreamBankEros_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area, CNI_0, AntMoist_0,
                                      Grow_0,
                                      CNP_0, Imper, ISRR, ISRA, CN, UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap,
                                      SatStor_0, RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                      TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal, NumAnimals,
                                      AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF, AvSlope, SedAAdjust,
                                      StreamLength)
    if (n42b > 0):
        return (n46c / n42b) * streambankeros * n85d
    else:
        return zeros((NYrs, 12))



from .VariableUnitTest import VariableUnitTest


class TestGRLostBarnNSum(VariableUnitTest):
    def test_GRLostBarnNSum(self):
        pass
        # z = self.z
        # np.testing.assert_array_almost_equal(
        #     GRLostBarnNSum.GRLostBarnNSum_2(),
        #     GRLostBarnNSum.GRLostBarnNSum(), decimal=7)



from numba.pycc import CC
from numpy import zeros

cc = CC('InitSnow_f_inner_compiled')


@cc.export('InitSnow_f_inner', '(int64, int64[:,::1], int64, float64[:,:,::1], float64[:,:,::1])')
def InitSnow_f_inner(NYrs, DaysMonth, InitSnow_0, Temp, Prec):
    result = zeros((NYrs, 12, 31))
    yesterday = InitSnow_0
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                if Temp[Y][i][j] <= 0:
                    result[Y][i][j] = yesterday + Prec[Y][i][j]
                else:
                    if yesterday > 0.001:
                        result[Y][i][j] = max(yesterday - 0.45 * Temp[Y][i][j], 0)
                    else:
                        result[Y][i][j] = yesterday
                yesterday = result[Y][i][j]
    return result



from numpy import sum

from .NGLoadN import NGLoadN
from .NGLoadN import NGLoadN_f
from gwlfe.Input.Animals.GrazingAnimal import GrazingAnimal
from gwlfe.Memoization import memoize
from gwlfe.enums import YesOrNo


def InitNgN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = 0
    ng_load_n = NGLoadN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    grazing_animal = GrazingAnimal(GrazingAnimal_0)
    for a in range(9):
        if grazing_animal[a] is YesOrNo.NO:
            result += ng_load_n[a]
    return result


@memoize
def InitNgN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    ng_load_n = NGLoadN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    return sum(ng_load_n)



from numpy import zeros

from gwlfe.Output.Loading.StreamBankN import StreamBankN
from gwlfe.Output.Loading.StreamBankN import StreamBankN_f


def NSTAB(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
          CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
          UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
          RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
          TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
          NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
          AvSlope, SedAAdjust, StreamLength, n42b, n46c, SedNitr, BankNFrac, n69c):
    result = zeros((NYrs, 12))
    streabank_n = StreamBankN(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                              CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                              UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                              RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                              TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                              NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                              AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac)

    for Y in range(NYrs):
        for i in range(12):
            result[Y][i] = 0
            if n42b > 0:
                result[Y][i] = (n46c / n42b) * streabank_n[Y][i] * n69c
    return result


def NSTAB_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
            CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
            UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
            RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
            TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
            NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
            AvSlope, SedAAdjust, StreamLength, n42b, n46c, SedNitr, BankNFrac, n69c):
    if n42b > 0:
        return (n46c / n42b) * StreamBankN_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, Area,
                                             CNI_0, AntMoist_0, Grow_0, CNP_0, Imper, ISRR, ISRA, CN,
                                             UnsatStor_0, KV, PcntET, DayHrs, MaxWaterCap, SatStor_0,
                                             RecessionCoef, SeepCoef, Qretention, PctAreaInfil, n25b, Landuse,
                                             TileDrainDensity, PointFlow, StreamWithdrawal, GroundWithdrawal,
                                             NumAnimals, AvgAnimalWt, StreamFlowVolAdj, SedAFactor_0, AvKF,
                                             AvSlope, SedAAdjust, StreamLength, SedNitr, BankNFrac) * n69c
    else:
        return zeros((NYrs, 12))



from numpy import zeros

from .InitGrN import InitGrN
from .InitGrN import InitGrN_f
from gwlfe.Memoization import memoize


def GrazingN(PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = zeros((12,))
    init_gr_n = InitGrN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    for i in range(12):
        result[i] = PctGrazing[i] * (init_gr_n / 12)
    return result


@memoize
def GrazingN_f(PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    return (PctGrazing * (InitGrN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN) / 12))[None, :]



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.nonGrazingAnimals.Losses import NGLostManN


class TestNGLostManN(VariableUnitTest):

    def test_NGLostManN(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NGLostManN.NGLostManN(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                  z.NGAppNRate, z.Prec, z.DaysMonth, z.NGPctSoilIncRate),
            NGLostManN.NGLostManN_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt,
                                    z.AnimalDailyN, z.NGAppNRate, z.Prec, z.DaysMonth,
                                    z.NGPctSoilIncRate), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.AFOS.GrazingAnimals.Losses import AvGRLostBarnNSum


class TestAvGRLostBarnNSum(VariableUnitTest):

    def test_AvGRLostBarnNSum(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvGRLostBarnNSum.AvGRLostBarnNSum_f(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                                z.GRPctManApp, z.PctGrazing, z.GRBarnNRate,
                                                z.Prec, z.DaysMonth, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                                                z.RunConCoeffN),
            AvGRLostBarnNSum.AvGRLostBarnNSum(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                              z.GRPctManApp, z.PctGrazing, z.GRBarnNRate,
                                              z.Prec, z.DaysMonth, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                                              z.RunConCoeffN), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.AvAnimalNSum import AvAnimalNSum_1


class TestAvAnimalNSum_1(VariableUnitTest):

    def test_AvAnimalNSum_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            AvAnimalNSum_1.AvAnimalNSum_1_f(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                            z.NGAppNRate, z.NGPctSoilIncRate, z.GRAppNRate,
                                            z.GRPctSoilIncRate, z.GrazingNRate, z.GRPctManApp, z.PctGrazing,
                                            z.GRBarnNRate,
                                            z.Prec, z.DaysMonth, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                                            z.RunConCoeffN, z.n41b, z.n85h, z.NGPctManApp,
                                            z.AWMSNgPct, z.NGBarnNRate, z.NgAWMSCoeffN, z.n41d, z.n85j, z.n41f, z.n85l,
                                            z.PctStreams, z.n42, z.n45, z.n69, z.n43, z.n64),
            AvAnimalNSum_1.AvAnimalNSum_1(z.NYrs, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                                          z.NGAppNRate, z.NGPctSoilIncRate, z.GRAppNRate,
                                          z.GRPctSoilIncRate, z.GrazingNRate, z.GRPctManApp, z.PctGrazing,
                                          z.GRBarnNRate,
                                          z.Prec, z.DaysMonth, z.AWMSGrPct, z.GrAWMSCoeffN, z.RunContPct,
                                          z.RunConCoeffN, z.n41b, z.n85h, z.NGPctManApp,
                                          z.AWMSNgPct, z.NGBarnNRate, z.NgAWMSCoeffN, z.n41d, z.n85j, z.n41f, z.n85l,
                                          z.PctStreams, z.n42, z.n45, z.n69, z.n43, z.n64), decimal=7)



from numpy import round
from numpy import zeros

from gwlfe.Input.LandUse.NLU import NLU
# from Timer import time_function
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.AttenN import AttenN
from gwlfe.MultiUse_Fxns.Runoff.RetentFactorN import RetentFactorN
from gwlfe.Output.Loading.LuTotNitr import LuTotNitr
from gwlfe.Output.Loading.LuTotNitr import LuTotNitr_f


def LuTotNitr_1(NYrs, NRur, NUrb, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, CN, Grow_0,
                Area, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth,
                FirstManureMonth2, LastManureMonth2, SedDelivRatio_0, KF, LS, C, P, SedNitr, CNP_0, Imper, ISRR, ISRA,
                Qretention, PctAreaInfil, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed, FilterWidth, PctStrmBuf, Acoef,
                CNI_0, Nqual, ShedAreaDrainLake, RetentNLake, AttenFlowDist, AttenFlowVel, AttenLossRateN):
    nlu = NLU(NRur, NUrb)
    result = zeros((NYrs, nlu))
    lu_tot_nitr = LuTotNitr(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0,
                            Area, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth,
                            FirstManureMonth2, LastManureMonth2, SedDelivRatio_0, KF, LS, C, P, SedNitr, CNP_0, Imper,
                            ISRR, ISRA, Qretention, PctAreaInfil, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed,
                            FilterWidth, PctStrmBuf, Acoef, CNI_0, Nqual)
    retent_factor_n = RetentFactorN(ShedAreaDrainLake, RetentNLake)
    atten_n = AttenN(AttenFlowDist, AttenFlowVel, AttenLossRateN)
    for y in range(NYrs):
        for l in range(nlu):
            result[y][l] = round((lu_tot_nitr[y][l] * retent_factor_n * (1 - atten_n)))
    return result


@memoize
def LuTotNitr_1_f(NYrs, NRur, NUrb, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, CN, Grow_0,
                  Area, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth,
                  FirstManureMonth2, LastManureMonth2, SedDelivRatio_0, KF, LS, C, P, SedNitr, CNP_0, Imper, ISRR, ISRA,
                  Qretention, PctAreaInfil, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed, FilterWidth, PctStrmBuf, Acoef,
                  CNI_0, Nqual, ShedAreaDrainLake, RetentNLake, AttenFlowDist, AttenFlowVel, AttenLossRateN):
    lu_tot_nitr = LuTotNitr_f(NYrs, DaysMonth, InitSnow_0, Temp, Prec, AntMoist_0, NRur, NUrb, CN, Grow_0,
                              Area, NitrConc, ManNitr, ManuredAreas, FirstManureMonth, LastManureMonth,
                              FirstManureMonth2, LastManureMonth2, SedDelivRatio_0, KF, LS, C, P, SedNitr, CNP_0, Imper,
                              ISRR, ISRA,
                              Qretention, PctAreaInfil, LoadRateImp, LoadRatePerv, Storm, UrbBMPRed, FilterWidth,
                              PctStrmBuf, Acoef,
                              CNI_0, Nqual)
    retent_factor_n = RetentFactorN(ShedAreaDrainLake, RetentNLake)
    atten_n = AttenN(AttenFlowDist, AttenFlowVel, AttenLossRateN)
    # TODO: this is only a temporary fix until WriteOutputfiles has been fully extracted
    # result[:, :NRur] = np.round((lu_tot_nitr * retent_factor_n * (1 - atten_n)))
    result = round((lu_tot_nitr * retent_factor_n * (1 - atten_n)))
    return result



from gwlfe.MultiUse_Fxns.Constants import NPConvert
from gwlfe.Output.AvAnimalNSum.N7b import N7b
from gwlfe.Output.AvAnimalNSum.N7b import N7b_f


def N7b_1(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
          GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
          GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN, n41d,
          n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    n7b = N7b(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
              GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
              GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN,
              n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64)
    result = n7b * NPConvert
    return result


def N7b_1_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
            GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
            GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN, n41d,
            n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64):
    n7b = N7b_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGAppNRate, NGPctSoilIncRate, GRAppNRate,
              GRPctSoilIncRate, GrazingNRate, GRPctManApp, PctGrazing, GRBarnNRate, Prec, DaysMonth, AWMSGrPct,
              GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h, NGPctManApp, AWMSNgPct, NGBarnNRate, NgAWMSCoeffN,
              n41d, n85j, n41f, n85l, PctStreams, n42, n45, n69, n43, n64)
    result = n7b * NPConvert
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Runoff import CNumPerv


class TestCNumPerv(VariableUnitTest):

    # @skip("not ready")
    def test_CNumPerv(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            CNumPerv.CNumPerv_f(z.NYrs, z.DaysMonth, z.Temp, z.NRur, z.NUrb, z.CNP_0, z.InitSnow_0, z.Prec, z.Grow_0,
                                z.AntMoist_0),
            CNumPerv.CNumPerv(z.NYrs, z.DaysMonth, z.Temp, z.NRur, z.NUrb, z.CNP_0, z.InitSnow_0, z.Prec, z.Grow_0,
                              z.AntMoist_0), decimal=7)



from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN_f
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import AvGRStreamN
from gwlfe.AFOS.GrazingAnimals.Losses.GRStreamN import AvGRStreamN_f
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum_f
from gwlfe.Output.AvAnimalNSum.AvAnimalNSum import AvAnimalNSum
from gwlfe.Output.AvAnimalNSum.AvAnimalNSum import AvAnimalNSum_f


def NAGBUFFER(n42, n43, n64, NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
              Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, GRPctManApp, PctGrazing, GRBarnNRate,
              AWMSGrPct, GrAWMSCoeffN, PctStreams, NGAppNRate, NGPctSoilIncRate, GRAppNRate, GRPctSoilIncRate,
              GrazingNRate):
    nglbn = AvNGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                             Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    grlbn = GRLBN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                  Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    grsn = AvGRStreamN(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
    av_animal_n_sum = AvAnimalNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                   NGAppNRate,
                                   Prec, DaysMonth, NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate,
                                   NGBarnNRate, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, PctGrazing,
                                   GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate)
    if n42 > 0:
        result = (n43 / n42) * n64 * (av_animal_n_sum - (nglbn + grlbn + grsn))
    else:
        result = 0
    return result


def NAGBUFFER_f(n42, n43, n64, NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, GRPctManApp, PctGrazing,
                GRBarnNRate,
                AWMSGrPct, GrAWMSCoeffN, PctStreams, NGAppNRate, NGPctSoilIncRate, GRAppNRate, GRPctSoilIncRate,
                GrazingNRate):
    if n42 > 0:
        nglbn = AvNGLostBarnNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                   NGBarnNRate,
                                   Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
        grlbn = GRLBN_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing,
                        GRBarnNRate,
                        Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
        grsn = AvGRStreamN_f(PctStreams, PctGrazing, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN)
        av_animal_n_sum = AvAnimalNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN,
                                         NGAppNRate,
                                         Prec, DaysMonth, NGPctSoilIncRate, GRPctManApp, GRAppNRate, GRPctSoilIncRate,
                                         NGBarnNRate, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, PctGrazing,
                                         GRBarnNRate, AWMSGrPct, GrAWMSCoeffN, PctStreams, GrazingNRate)

        return (n43 / n42) * n64 * (av_animal_n_sum - (nglbn + grlbn + grsn))
    else:
        return 0



from numpy import zeros

from gwlfe.Input.Animals.GrazingAnimal import GrazingAnimal
from gwlfe.Input.Animals.GrazingAnimal import GrazingAnimal_f
from gwlfe.MultiUse_Fxns.Constants import NAnimals
from gwlfe.enums import YesOrNo


def GRLoadN(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    result = zeros((NAnimals,))
    grazing_animal = GrazingAnimal(GrazingAnimal_0)
    for a in range(9):
        if grazing_animal[a] is YesOrNo.NO:
            pass
        elif grazing_animal[a] is YesOrNo.YES:
            result[a] = (NumAnimals[a] * AvgAnimalWt[a] / 1000) * AnimalDailyN[a] * 365
    return result


def GRLoadN_f(GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN):
    grazing_animals = GrazingAnimal_f(GrazingAnimal_0)
    return (NumAnimals[grazing_animals] * AvgAnimalWt[grazing_animals] / 1000) * AnimalDailyN[grazing_animals] * 365



from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum
from gwlfe.AFOS.nonGrazingAnimals.Losses.NGLostBarnN import AvNGLostBarnNSum_f


def NAWMSP(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
           Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j):
    nglbn = AvNGLostBarnNSum(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
                             Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41d / 100) * n85j * nglbn
    return result


def NAWMSP_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, NGBarnNRate,
             Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN, n41d, n85j):
    return (n41d / 100) * n85j * AvNGLostBarnNSum_f(NYrs, NGPctManApp, GrazingAnimal_0, NumAnimals, AvgAnimalWt,
                                                    AnimalDailyN, NGBarnNRate,
                                                    Prec, DaysMonth, AWMSNgPct, NgAWMSCoeffN, RunContPct, RunConCoeffN)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Input.WaterBudget import ET_1


class TestET_f(VariableUnitTest):
    def test_ET_f(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            ET_1.ET_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                        z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                        z.DayHrs, z.MaxWaterCap),
            ET_1.ET_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                      z.AntMoist_0, z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                      z.DayHrs, z.MaxWaterCap, z.ETFlag), decimal=7)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.MultiUse_Fxns.Erosion import Erosion_1


class TestErosion_f(VariableUnitTest):

    def test_elementwise_Erosion(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath+"/Erosion.npy"),
            Erosion_1.Erosion_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0,
                                z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                z.DayHrs,
                                z.MaxWaterCap, z.SatStor_0,
                                z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                z.TileDrainDensity, z.PointFlow,
                                z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                z.SedAFactor_0,
                                z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                z.n42,
                                z.n45, z.n85, z.UrbBankStab,
                                z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P), decimal=7)

    def test_Erosion_1(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            Erosion_1.Erosion_1_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                  z.AntMoist_0,
                                  z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                  z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                  z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                  z.TileDrainDensity, z.PointFlow,
                                  z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt,
                                  z.StreamFlowVolAdj, z.SedAFactor_0,
                                  z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                  z.n42, z.n45, z.n85, z.UrbBankStab,
                                  z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P),
            Erosion_1.Erosion_1(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.NRur, z.NUrb, z.Area, z.CNI_0,
                                z.AntMoist_0,
                                z.Grow_0, z.CNP_0, z.Imper, z.ISRR, z.ISRA, z.CN, z.UnsatStor_0, z.KV, z.PcntET,
                                z.DayHrs, z.MaxWaterCap, z.SatStor_0,
                                z.RecessionCoef, z.SeepCoef, z.Qretention, z.PctAreaInfil, z.n25b, z.Landuse,
                                z.TileDrainDensity, z.PointFlow,
                                z.StreamWithdrawal, z.GroundWithdrawal, z.NumAnimals, z.AvgAnimalWt, z.StreamFlowVolAdj,
                                z.SedAFactor_0,
                                z.AvKF, z.AvSlope, z.SedAAdjust, z.StreamLength, z.n42b, z.n46c, z.n85d, z.AgLength,
                                z.n42, z.n45, z.n85, z.UrbBankStab,
                                z.SedDelivRatio_0, z.Acoef, z.KF, z.LS, z.C, z.P), decimal=7)



from numpy import sum
from numpy import zeros

from gwlfe.Input.LandUse.AreaTotal import AreaTotal
from gwlfe.Input.LandUse.RurAreaTotal import RurAreaTotal
# from Timer import time_function
from gwlfe.Input.WaterBudget.Water import Water
from gwlfe.Memoization import memoize
from gwlfe.MultiUse_Fxns.Runoff.Qrun import Qrun, Qrun_f
from gwlfe.MultiUse_Fxns.Runoff.Retention import Retention


# @time_function
@memoize
def RuralQTotal(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, NUrb, AntMoist_0, Grow_0, Area):
    result = zeros((NYrs, 12, 31))
    water = Water(NYrs, DaysMonth, InitSnow_0, Temp, Prec)
    q_run = Qrun(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)
    rur_area_total = RurAreaTotal(NRur, Area)
    retention = Retention(NYrs, DaysMonth, Temp, Prec, InitSnow_0, AntMoist_0, NRur, NUrb, CN, Grow_0)
    area_total = AreaTotal(NRur, NUrb, Area)
    for Y in range(NYrs):
        for i in range(12):
            for j in range(DaysMonth[Y][i]):
                result[Y][i][j] = 0  # this does not need to be calculated daily
                if Temp[Y][i][j] > 0 and water[Y][i][j] > 0.01:
                    for l in range(NRur):
                        if CN[l] > 0:  # TODO: CN in set to all zeros in datamodel:42
                            if water[Y][i][j] >= 0.2 * retention[Y][i][j][l]:
                                result[Y][i][j] += q_run[Y][i][j][l] * Area[l] / rur_area_total
                    if result[Y][i][j] > 0:
                        result[Y][i][j] *= rur_area_total / area_total
                    else:
                        result[Y][i][j] = 0  # TODO: this seems redundant
                else:
                    pass
    return result


@memoize
def RuralQTotal_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, CN, NUrb, AntMoist_0, Grow_0, Area):
    result = zeros((NYrs, 12, 31))
    q_run = Qrun_f(NYrs, DaysMonth, Temp, InitSnow_0, Prec, NRur, NUrb, CN, AntMoist_0, Grow_0)
    area_total = AreaTotal(NRur, NUrb, Area)
    qrun_area = q_run * Area
    result = sum(qrun_area, axis=3) / area_total
    return result



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.Output.Loading import SedYield


class TestSedYield(VariableUnitTest):

    def test_elementwise_SedYield(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            np.load(self.basepath + "/SedYield.npy"),
            SedYield.SedYield(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C,
                              z.P, z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention,
                              z.PctAreaInfil, z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0), decimal=7)

    def test_SedYield(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            SedYield.SedYield_f(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C,
                                z.P, z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention,
                                z.PctAreaInfil, z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0),
            SedYield.SedYield(z.NYrs, z.DaysMonth, z.Temp, z.InitSnow_0, z.Prec, z.Acoef, z.NRur, z.KF, z.LS, z.C,
                              z.P, z.Area, z.NUrb, z.CNI_0, z.AntMoist_0, z.Grow_0, z.ISRR, z.ISRA, z.Qretention,
                              z.PctAreaInfil, z.n25b, z.CN, z.CNP_0, z.Imper, z.SedDelivRatio_0), decimal=7)



from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN
from gwlfe.AFOS.GrazingAnimals.Losses.GRLBN import GRLBN_f


def NAWMSL(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
           Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h):
    grlbn = GRLBN(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
                  Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)
    result = (n41b / 100) * n85h * grlbn
    return result


def NAWMSL_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp, PctGrazing, GRBarnNRate,
             Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN, n41b, n85h):
    return (n41b / 100) * n85h * GRLBN_f(NYrs, GrazingAnimal_0, NumAnimals, AvgAnimalWt, AnimalDailyN, GRPctManApp,
                                         PctGrazing, GRBarnNRate,
                                         Prec, DaysMonth, AWMSGrPct, GrAWMSCoeffN, RunContPct, RunConCoeffN)



import numpy as np

from .VariableUnitTest import VariableUnitTest
from gwlfe.BMPs.AgAnimal import NAWMSP


class TestNAWMSP(VariableUnitTest):

    def test_NAWMSP(self):
        z = self.z
        np.testing.assert_array_almost_equal(
            NAWMSP.NAWMSP_f(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                            z.NGBarnNRate,
                            z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN, z.n41d,
                            z.n85j),
            NAWMSP.NAWMSP(z.NYrs, z.NGPctManApp, z.GrazingAnimal_0, z.NumAnimals, z.AvgAnimalWt, z.AnimalDailyN,
                          z.NGBarnNRate,
                          z.Prec, z.DaysMonth, z.AWMSNgPct, z.NgAWMSCoeffN, z.RunContPct, z.RunConCoeffN, z.n41d,
                          z.n85j), decimal=7)
